#include "huc.h"


/* VRAM locations */

#define VRAM_SELECT     0x3400

#define VRAM_WHITE1     0x3000
#define VRAM_WHITE2     0x3010
#define VRAM_WHITE3     0x3020
#define VRAM_WHITE4     0x3030

#define VRAM_BLACK1     0x3040
#define VRAM_BLACK2     0x3050
#define VRAM_BLACK3     0x3060
#define VRAM_BLACK4     0x3070

#define VRAM_EMPTY1     0x3080
#define VRAM_EMPTY2     0x3090
#define VRAM_EMPTY3     0x3100
#define VRAM_EMPTY4     0x3110

#define VRAM_TITLE1     0x5000
#define VRAM_TITLE2     0x5200
#define VRAM_TITLE3     0x5400
#define VRAM_TITLE4     0x5600


/* miscellaneous constants */

#define BGPAL           0
#define BGPAL1          1
#define SPRPAL          0
#define TITPAL          2


#define X_BOARD_OFFSET  8
#define Y_BOARD_OFFSET  8
#define X_SPR_OFFSET    64
#define Y_SPR_OFFSET    64
#define CHARS_PER_LINE  64

#define LOOK_NEUT   0
#define LOOK_LEFT   1
#define LOOK_RIGHT  2
#define LOOK_UP     1
#define LOOK_DOWN   2

#define PLAYER_WHITE  1
#define PLAYER_BLACK  2


/* include graphics */

#incchr(white1, "flipit.pcx", 0,16, 1, 1);
#incchr(white2, "flipit.pcx", 8,16, 1, 1);
#incchr(white3, "flipit.pcx", 0,24, 1, 1);
#incchr(white4, "flipit.pcx", 8,24, 1, 1);

#incchr(black1, "flipit.pcx", 0,32, 1, 1);
#incchr(black2, "flipit.pcx", 8,32, 1, 1);
#incchr(black3, "flipit.pcx", 0,40, 1, 1);
#incchr(black4, "flipit.pcx", 8,40, 1, 1);

#incchr(empty1, "flipit.pcx",16, 0, 1, 1);
#incchr(empty2, "flipit.pcx",24, 0, 1, 1);
#incchr(empty3, "flipit.pcx",16, 8, 1, 1);
#incchr(empty4, "flipit.pcx",24, 8, 1, 1);

#incspr(select,"flipit.pcx", 0, 0, 1, 1);

#incspr(wanim1,"flipit.pcx", 0,16, 1, 1);
#incspr(wanim2,"flipit.pcx",16,16, 1, 1);
#incspr(wanim3,"flipit.pcx",32,16, 1, 1);
#incspr(wanim4,"flipit.pcx",48,16, 1, 1);
#incspr(wanim5,"flipit.pcx",64,16, 1, 1);
#incspr(wanim6,"flipit.pcx",80,16, 1, 1);
#incspr(wanim7,"flipit.pcx",96,16, 1, 1);
#incspr(wanim8,"flipit.pcx",16, 0, 1, 1);
#incspr(wanim9,"flipit.pcx",96,32, 1, 1);
#incspr(wanima,"flipit.pcx",80,32, 1, 1);
#incspr(wanimb,"flipit.pcx",64,32, 1, 1);
#incspr(wanimc,"flipit.pcx",48,32, 1, 1);
#incspr(wanimd,"flipit.pcx",32,32, 1, 1);
#incspr(wanime,"flipit.pcx",16,32, 1, 1);
#incspr(wanimf,"flipit.pcx", 0,32, 1, 1);

#incpal(colors, "flipit.pcx", 0, 2);



/* GLOBALS */

/* attributes for each square on board */

char position[64];
char changed[64];
char priority[64];

/* hard level priority adjustment */

const char prio_adjust[64] = {
  5, 0, 4, 4, 4, 4, 0, 5,
  0, 0, 1, 1, 1, 1, 0, 0,
  4, 1, 3, 3, 3, 3, 1, 4,
  4, 1, 3, 3, 3, 3, 1, 4,
  4, 1, 3, 3, 3, 3, 1, 4,
  4, 1, 3, 3, 3, 3, 1, 4,
  0, 0, 1, 1, 1, 1, 0, 0,
  5, 0, 4, 4, 4, 4, 0, 5
};

/* scores */

char total1;
char total2;

/* selector position */

char ptrx;
char ptry;

/* Misc */

char selected;
char player;
char computer;

char hard;

char maxprio;
char maxpos;


/* --------- START FLIPIT ---------- */

/* display score */

dispscore()
{
char i;

    put_string("White", 0x140);
    put_string("Black", 0x159);

    put_string("Score", 0x2C0);
    put_string("Score", 0x2D9);

    if (player == PLAYER_WHITE)
    {
        put_string("Players ", 0x1C0);
        put_string(" Turn   ", 0x200);
        put_string("        ", 0x1D9);
        put_string("        ", 0x219);
    }
    else if (player == PLAYER_BLACK)
    {
        put_string("        ", 0x1C0);
        put_string("        ", 0x200);

	if (computer == 0)
	{
            put_string("Players ", 0x1D9);
            put_string(" Turn   ", 0x219);
	}
	else
	{
            put_string("Thinking", 0x1D9);
	}
    }
    else
    {
        put_string("        ", 0x1C0);
        put_string("        ", 0x200);
        put_string("        ", 0x1D9);
        put_string("        ", 0x219);
    }

    total1 = 0;
    total2 = 0;

    for (i=0; i<64; i++)
    {
        switch (position[i])
	{
        case PLAYER_WHITE:
            total1++;
            break;

	case PLAYER_BLACK:
	    total2++;
	    break;
	}
    }

/*
    put_number(total1, 2, 1, 8);
    put_number(total2, 2, 20, 8);
*/

    put_number(total1, 2, 0x341);
    put_number(total2, 2, 0x35A);

}


/* position player's selection pointer */

ptr_move()
{
    spr_set(0);

    if (ptrx < 8)
        spr_x(X_SPR_OFFSET + (ptrx<<4) );
    else
        spr_x(0);

    if (ptry < 8)
        spr_y(Y_SPR_OFFSET + (ptry<<4) );
    else
        spr_y(0);

    spr_pattern(VRAM_SELECT);
    spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
    spr_pal(SPRPAL);
    spr_pri(1);
    satb_update();
}


/* display an individual square */

display(pos)
char pos;
{
char x,y,temp;
int bataddr;

    x = pos & 7;
    y = pos >> 3;

    bataddr = ((y<<1)+Y_BOARD_OFFSET) * CHARS_PER_LINE + (x<<1) + X_BOARD_OFFSET;

    temp = position[pos];

    if (temp == PLAYER_WHITE)
    {
        put_raw( (BGPAL1<<12)|(VRAM_WHITE1>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_WHITE2>>4), bataddr);

	bataddr += CHARS_PER_LINE-1;

        put_raw( (BGPAL1<<12)|(VRAM_WHITE3>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_WHITE4>>4), bataddr);
    }

    else if (temp == PLAYER_BLACK)
    {
        put_raw( (BGPAL1<<12)|(VRAM_BLACK1>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_BLACK2>>4), bataddr);

	bataddr += CHARS_PER_LINE-1;

        put_raw( (BGPAL1<<12)|(VRAM_BLACK3>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_BLACK4>>4), bataddr);
    }

    else
    {
        put_raw( (BGPAL1<<12)|(VRAM_EMPTY1>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_EMPTY2>>4), bataddr);

	bataddr += CHARS_PER_LINE-1;

        put_raw( (BGPAL1<<12)|(VRAM_EMPTY3>>4), bataddr++);
        put_raw( (BGPAL1<<12)|(VRAM_EMPTY4>>4), bataddr);
    }
}


displayall()
{
char i;

    for (i=0; i<64; i++)
    {
        display(i);
	changed[i] = 0;
    }
}


displaydirty()
{
char i;

    for (i=0; i<64; i++)
    {
        if (changed[i] != 0)
	{
            display(i);
	    changed[i] = 0;
	}
    }
}


disppriority()
{
char i,x,y;
int bataddr;

    for (i=0; i<64; i++)
    {
        y = i >> 3;
	x = i & 7;

	if (priority[i] != 0)
	{
            bataddr = ((y<<1)+Y_BOARD_OFFSET+1) * CHARS_PER_LINE + (x<<1) + X_BOARD_OFFSET;
            put_number(priority[i], 1, bataddr);
	}
    }
}


setpriority()
{
char i,x,y;

    for (i=0; i<64; i++)
    {
        y = i >> 3;
	x = i & 7;
        priority[i] = okmove(x,y);
    }
}


hardadjust_priority()
{
char i,x,y;

    for (i=0; i<64; i++)
    {
        if (priority[i] != 0)
	{
            priority[i] += prio_adjust[i];
	}
    }
}


/* return the value of the piece in the (x,y) location */

occupies(x, y)
char x,y;
{
char location;

    location = (y<<3) | x;
    return(position[location]);
}


adjustxy(x,y,dx,dy)
char *x;
char *y;
char dx,dy;
{
    switch (dx)
    {
    case LOOK_LEFT:
        if (*x == 0)
            return(1);

	(*x)--;
	break;

    case LOOK_RIGHT:
	if (*x == 7)
            return(1);

	(*x)++;
	break;
    }

    switch (dy)
    {
    case LOOK_UP:
        if (*y == 0)
            return(1);

	(*y)--;
	break;

    case LOOK_DOWN:
	if (*y == 7)
            return(1);

	(*y)++;
	break;
    }

    return(0);
}


/* count how many pieces can flip in this direction  */
/* given a start point (x,y) and a direction (dx,dy) */

testline(x,y,dx,dy)
char x,y,dx,dy;
{
char otherplay;
char flag;
char tmp;

    tmp = 0;

    if (player == PLAYER_WHITE)
        otherplay = PLAYER_BLACK;
    else
        otherplay = PLAYER_WHITE;

    flag = adjustxy(&x,&y,dx,dy);

    if (flag == 1)
        return(0);

    while ((occupies(x,y) == otherplay) && (flag == 0))
    {
        tmp++;
	flag = adjustxy(&x,&y,dx,dy);
    }

    if (flag == 1)
        return(0);

    if (occupies(x,y) == player)
        return(tmp);

    return(0);
}


/* check how many pieces will flip for this move    */
/* in all directions.  Zero means move is not valid */

okmove(x,y)
char x,y;
{
char captured;

    captured = 0;

    if (occupies(x, y) != 0)
        return(0);

    captured += testline(x,y,LOOK_RIGHT,LOOK_NEUT);	/* right */
    captured += testline(x,y,LOOK_RIGHT,LOOK_DOWN);	/* diag dn-rt */
    captured += testline(x,y,LOOK_NEUT, LOOK_DOWN);	/* down */
    captured += testline(x,y,LOOK_LEFT, LOOK_DOWN);	/* diag dn-lft */
    captured += testline(x,y,LOOK_LEFT, LOOK_NEUT);	/* left */
    captured += testline(x,y,LOOK_LEFT, LOOK_UP);	/* diag up-lft */
    captured += testline(x,y,LOOK_NEUT, LOOK_UP);	/* up */
    captured += testline(x,y,LOOK_RIGHT,LOOK_UP);	/* diag up-rt */

    return(captured);
}


/* actually execute the move - in one direction */

cvtline(x,y,dx,dy)
char x,y;
int dx,dy;
{
char otherplay;
char flag;
char tmp;
char location;

    tmp = 0;

    if (player == PLAYER_WHITE)
        otherplay = PLAYER_BLACK;
    else
        otherplay = PLAYER_WHITE;

    flag = adjustxy(&x,&y,dx,dy);

    if (flag == 1)
        return(0);

    while ((occupies(x,y) == otherplay) && (flag == 0))
    {
        tmp++;
	location = (y<<3) | x;
	position[location] = player;
	changed[location]  = 1;
	flag = adjustxy(&x,&y,dx,dy);
    }

    if (flag == 1)
        return(0);

    if (occupies(x,y) == player)
        return(tmp);

    return(0);
}


/* actually execute the move - direction by direction */

domove(x,y)
char x,y;
{
char captured;
char location;

    captured = 0;

    location = (y<<3) + x;

    position[location] = player;
    changed[location] = 1;

    if (testline(x,y,LOOK_RIGHT,LOOK_NEUT) > 0)		/* right */
         cvtline(x,y,LOOK_RIGHT,LOOK_NEUT);

    if (testline(x,y,LOOK_RIGHT,LOOK_DOWN) > 0)		/* diag dn-rt */
         cvtline(x,y,LOOK_RIGHT,LOOK_DOWN);
    
    if (testline(x,y,LOOK_NEUT, LOOK_DOWN) > 0)		/* down */
         cvtline(x,y,LOOK_NEUT, LOOK_DOWN);

    if (testline(x,y,LOOK_LEFT, LOOK_DOWN) > 0)		/* diag dn-lft */
         cvtline(x,y,LOOK_LEFT, LOOK_DOWN);

    if (testline(x,y,LOOK_LEFT, LOOK_NEUT) > 0)		/* left */
         cvtline(x,y,LOOK_LEFT, LOOK_NEUT);

    if (testline(x,y,LOOK_LEFT, LOOK_UP) > 0)		/* diag up-lft */
         cvtline(x,y,LOOK_LEFT, LOOK_UP);

    if (testline(x,y,LOOK_NEUT, LOOK_UP) > 0)		/* up */
         cvtline(x,y,LOOK_NEUT, LOOK_UP);

    if (testline(x,y,LOOK_RIGHT,LOOK_UP) > 0)		/* diag up-rt */
         cvtline(x,y,LOOK_RIGHT,LOOK_UP);
}


computermove()
{
char i;

    if (hard == 1)
    {
      hardadjust_priority();
    }

    maxprio = 0;

    for (i=0; i<64; i++)
    {
        if (priority[i] > maxprio)
	{
            maxprio = priority[i];
	    maxpos = i;
	}
    }

    if (maxprio > 0)
    {
        ptrx = maxpos & 7;
	ptry = maxpos >> 3;
    }

    return(maxprio);
}


/* execute the game from setup board until endgame */

playgame()
{
char i,move;
char location;

    cls();

    init_satb();

    set_sprpal(SPRPAL, colors, 2);
    set_bgpal(BGPAL, colors, 2);


/* set up playing surface */

    for (i=0; i<64; i++)
    {
        position[i] = 0;
	changed[i]  = 1;
    }
    position[(3<<3) + 3] = 1;
    position[(3<<3) + 4] = 2;
    position[(4<<3) + 3] = 2;
    position[(4<<3) + 4] = 1;

    displayall();

/* start with white's turn */

    player = PLAYER_WHITE;
    dispscore();

/* initial pointer position */

    ptrx = 3;
    ptry = 5;

/* verify # of good moves in each */
/* position, for current player   */

    setpriority();

    fade_in(0,15,colors);

/* MAIN LOOP */

    while (1)
    {
        vsync(0);
        satb_update();

	/* based on joystick delta, move pointer */

/*	move = joytrg(player);  */

	if ((computer == 1) && (player == PLAYER_BLACK))
	{
            selected = computermove();

            if (selected > 0)
            {
	        /* reposition pointer */
	        vsync(30);
                ptr_move();
	        vsync(30);
                domove(ptrx, ptry);
            }
	}
	else
	{
	    move = joytrg(0);

	    if (move & JOY_LEFT)
	    {
                if (ptrx > 0)
                    ptrx--;
	    }
	    if (move & JOY_RGHT)
	    {
                if (ptrx < 7)
                    ptrx++;
	    }
	    if (move & JOY_UP)
	    {
                if (ptry > 0)
                    ptry--;
	    }
	    if (move & JOY_DOWN)
	    {
                if (ptry < 7)
                    ptry++;
	    }
	    if (move & JOY_A)
	    {
                location = (ptry<<3) | ptrx;
                selected = priority[location];
	        if (selected > 0)
                    domove(ptrx,ptry);
	    }
	    if (move & JOY_SLCT)
	    {
                if ( (joy(0) & (JOY_B|JOY_SLCT)) == (JOY_B|JOY_SLCT) )
	        {
                    disppriority();
                    while ( (joy(0) & (JOY_B|JOY_SLCT)) == (JOY_B|JOY_SLCT) );
                    displayall();
	        }
	    }
	}

	/* reposition pointer */

        ptr_move();

	/* if move taken, display squares that have flipped */

	if (selected != 0)
	{
	    selected = 0;

	    if (player == PLAYER_WHITE)
                player = PLAYER_BLACK;
	    else
                player = PLAYER_WHITE;

	    displaydirty();
	    dispscore();

            /* update # of good moves in each */
	    /* position, for current player   */

	    setpriority();

	    maxprio = 0;
	    for (i=0; i<64; i++)
	    {
                if (priority[i] > maxprio)
                    maxprio = priority[i];
	    }

	    if (maxprio == 0)
	    {
                /* player cannot make a move */
                /* switch player and check other player for end of game */

                if (player == PLAYER_WHITE)
                    player = PLAYER_BLACK;
                else
                    player = PLAYER_WHITE;

	        setpriority();

		maxprio = 0;
	        for (i=0; i<64; i++)
	        {
                    if (priority[i] > maxprio)
                        maxprio = priority[i];
	        }

		if (maxprio == 0)
		{
                /* end of game; determine winner */

		    player = 0;
		    dispscore();

                    put_string("     End of game", 0x146);

		    if (total1 > total2)
                        put_string("     White wins", 0x186);

                    else if (total1 < total2)
                        put_string("     Black wins", 0x186);

                    else
                        put_string("     Nobody wins", 0x186);

		    /* wait for return key */
		    while ((joytrg(0) & JOY_STRT) == 0);

                    /* hide the pointer */
                    spr_hide();
                    satb_update();


		    return(0);
		}
		else
		{

                    put_string("     Cannot move  ", 0x106);
                    put_string("    Press button A", 0x146);
                    put_string("     to continue  ", 0x186);
		    while ((joytrg(0) & JOY_A) == 0);
                    put_string("                  ", 0x106);
                    put_string("                  ", 0x146);
                    put_string("                  ", 0x186);
		}
	    }

	    displaydirty();
	    dispscore();
	}
    }
}


titlescreen()
{
char select;

    select = 0;
    computer = 0;
    hard = 0;

/* This will display the titlescreen soon */

    cls();


    put_string("FLIPIT", 0x28C);
    put_string("By Dave Shadoff", 0x307);

    put_digit(2, 0x489);
    put_string("Player", 0x48b);

    put_digit(1, 0x4c9);
    put_string("Player vs Com - Easy", 0x4cb);
    put_digit(1, 0x509);
    put_string("Player vs Com - Hard", 0x50b);

    fade_in(0,15,colors);

    select = 0;

    while (1)
    {
        vsync(0);

        if ( (joytrg(0) == JOY_DOWN) && (select < 2) )
            select++;
        else if ( (joytrg(0) == JOY_UP) && (select > 0) )
            select--;

        else if (joytrg(0) == JOY_STRT)
        {
            fade_out(0,15);
            return(0);
        }

        switch (select)
        {
	case 0:
            put_string("o", 0x487);
            put_string(" ", 0x4c7);
            put_string(" ", 0x507);
            computer = 0;
            hard = 0;
            break;

	case 1:
            put_string(" ", 0x487);
            put_string("o", 0x4c7);
            put_string(" ", 0x507);
            computer = 1;
            hard = 0;
            break;

	case 2:
            put_string(" ", 0x487);
            put_string(" ", 0x4c7);
            put_string("o", 0x507);
            computer = 1;
            hard = 1;
            break;
        }
    }

    ptrx = 8;
    ptry = 8;

    ptr_move();
}


fade_in(low_index,hi_index,pal)
int low_index,hi_index;
int *pal;
{
int g,final_g,index;
char r,final_r,b,final_b;
char count;

    count = 8;

    while (count)
    {
        for (index=low_index; index<=hi_index; index++)
        {
            g = get_color(index);

            final_g = colors[index - low_index];

            b = g & 7;
            final_b = final_g & 7;

            g >>= 3;
            final_g >>= 3;

            r = g & 7;
            final_r = final_g & 7;

            g >>= 3;
            final_g >>= 3;

            g &= 7;
            final_g &= 7;

            if (b<final_b) b++;
            else if (b>final_b) b--;

            if (g<final_g) g++;
            else if (g>final_g) g--;

            if (r<final_r) r++;
            else if (r>final_r) r--;

            set_color_rgb(index,r,g,b);
/*
        put_number(final_r, 2, 1, index-low_index);
        put_number(final_g, 2, 3, index-low_index);
        put_number(final_b, 2, 5, index-low_index);
*/
        }
        vsync(2);
/*
        while (joy(0));
        while (!joy(0));
*/
        count--;
    }

}


fade_out(low_index,hi_index)
int low_index,hi_index;
{
int g,index;
char r,b;
char count,cont;

    count = 8;
    cont = 1;

/*  while (count && cont) */
    while (count)
    {
        for (index=low_index; index<=hi_index; index++)
        {
            cont = g = get_color(index);

            b = g & 7;
            g >>= 3;
            r = g & 7;
            g >>= 3;
            g &= 7;

            if (b) b--;
            if (g) g--;
            if (r) r--;

            set_color_rgb(index,r,g,b);
        }
        vsync(2);
/*
        while (joy(0));
        while (!joy(0));
*/
        count--;
    }
}


main()
{

/* load some basic stuff */

    set_xres(256);

    set_sprpal(SPRPAL, colors, 2);
    set_bgpal(BGPAL, colors, 2);

/* computer can't play yet */

    computer = 0;

/* load character data for squares */

    load_vram(VRAM_WHITE1, white1, 16);
    load_vram(VRAM_WHITE2, white2, 16);
    load_vram(VRAM_WHITE3, white3, 16);
    load_vram(VRAM_WHITE4, white4, 16);

    load_vram(VRAM_BLACK1, black1, 16);
    load_vram(VRAM_BLACK2, black2, 16);
    load_vram(VRAM_BLACK3, black3, 16);
    load_vram(VRAM_BLACK4, black4, 16);

    load_vram(VRAM_EMPTY1, empty1, 16);
    load_vram(VRAM_EMPTY2, empty2, 16);
    load_vram(VRAM_EMPTY3, empty3, 16);
    load_vram(VRAM_EMPTY4, empty4, 16);


/* load data for selection pointer */

    load_sprites(VRAM_SELECT, select, 1);

    while (1)
    {
        titlescreen();
        playgame();
    }
}

